 /*******************************************************************************

    TERMOMETRO DIGITAL COM 4 DIGITOS

    (VERSO 7 SEGUIMENTOS NODO COMUM)
    
    MULTIPLEXAO INTERCALADA NA ROTINA
    
    SENSOR: DS18B20    (-55 a +125 C)

    AUTOR: CLAUDIO LRIOS

    DATA: 17/11/2022

    COM  AT89C2051

    OBS. MONTAGEM  DE NATUREZA DIDTICA
    USE COMPILADOR MIKROC FOR 8051 V.3.6.0

    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE
*******************************************************************************/
//BIT

sbit   a3    at  P1.B3; //nodo do display das centenas
sbit   a2    at  P1.B4; //nodo do display das dezenas
sbit   a1    at  P1.B5; //nodo do display das unidades
sbit   a0    at  P1.B6; //nodo do display das ps virgula
sbit OW_Bit at P1_0_bit;//linha do sensor DS18b20  (precisa pull up externo)

//variveis globais
char pv,un,dez,cent,vez; //registradores
unsigned int temp;//armazena valord da temperatura lida no DS18b20
bit f_neg;//flag de temperatura negativa

 //definir a resoluo a ser usada
const unsigned short TEMP_RESOLUTION = 12;


const char num[]= {
       0b11000000, //0   0xc0
       0b11111001, //1   0xf9
       0b01100100, //2   0x64
       0b01110000, //3   0x70
       0b01011001, //4   0x59
       0b01010010, //5   0x52
       0b01000010, //6   0x42
       0b11111000, //7   0xf8
       0b01000000, //8   0x40
       0b01010000, //9   0x50
       0b01111111, //'-' 0x7f
};


 /******************************************************************************
      Multiplexao dos displays de 7 seguimentos (nodo Comum)
 ******************************************************************************/
void multiplexa(){
    a3=1;
    a2=1;
    a1=1;
    a0=1;
    if(++vez>3)vez=0;
    if(vez==0){P3=pv; a0=0;}  //mostra ps virgula
    if(vez==1){P3=un; a1=0;}    //mostra unidades
    if(vez==2){P3=dez; a2=0;}   //mostra dezenas
    if(vez==3){P3=cent; a3=0;}  //mostra centenas

}
 /******************************************************************************
     Delay com Multiplexao dos displays de 7 seguimentos (nodo Comum)
 ******************************************************************************/
void delay_mpx( char valor){
  char d;
  for(d=0;d<valor;d++){
      multiplexa();
      delay_ms(1);
  }
}
 /******************************************************************************
      Rotina de converso e apresentao  nos displays
 ******************************************************************************/

void Mostra_Temp(unsigned int temp_lida) {
  const unsigned short RES_SHIFT = TEMP_RESOLUTION - 8;
  char temp_int;
  unsigned int temp_fraction;

  // verifica se a temperatura  negativa ou positiva
  if (temp_lida & 0x8000) {
    f_neg = 1;
    temp_lida = ~temp_lida + 1;
  }
  // extrai a parte inteira da temperatura
  temp_int = temp_lida >> RES_SHIFT;
  cent=0;
  while(temp_int>=100){temp_int= temp_int - 100; cent++;}
  dez=0;
  while(temp_int>=10){temp_int= temp_int - 10; dez++;}; //obtm as dezenas
  un=temp_int;//obtm as unidades
  
  // extrai a parte fracionaria (1 casa aps a virgula)e converte em inteiro
  temp_fraction  = temp_lida << (4-RES_SHIFT);
  temp_fraction &= 0x000F;
  temp_fraction *= 625; //multiplica por 625
  pv=0;
  while(temp_fraction>=1000){
    temp_fraction= temp_fraction - 1000; 
    pv++;
  }//obtem casa aps a virgula
  
  cent=num[cent];  //converte centenas para 7 seguimentos
  if(cent== 0xc0)cent=0xff; //apaga se for zero a esquerda
  dez=num[dez]; //converte dezena para 7 seguimentos
  if(cent==0xff && dez==0xc0)dez=0xff; //apaga dezenas com zeros a esquerda
  un=num[un];  //converte unidades para 7 seguimentos
  pv=num[pv];  //converte casa aps a virgula para 7 seguimentos
  if(f_neg){f_neg=0; cent=num[10];}//pem '-' na centena se temperatura  negativa
}

/*************************************************************************
       Rotina Main
*************************************************************************/
void main() {
     P1=0b00111011;  //desliga anodos
     P3=0xff;  //desliga  seguimentos
     vez=0;
     f_neg=0;//temperatura positiva
 /*************************************************************************
       Lao Infinito
*************************************************************************/

  for(;;){
    Ow_Reset();    // Onewire reset signal
    multiplexa();  //multiplexa displays
    Ow_Write(0xCC);// Envia comando SKIP_ROM
    multiplexa();  //multiplexa displays
    Ow_Write(0x44);// Envia comando CONVERT_T
    delay_mpx(120); //delay ps comando

    Ow_Reset();    // Onewire reset signal
    multiplexa();  //multiplexa displays
    Ow_Write(0xCC);// Envia comando SKIP_ROM
    multiplexa();  //multiplexa displays
    Ow_Write(0xBE);// Envia comando READ_SCRATCHPAD

    temp =  Ow_Read();//l temperatura (byte1)
    multiplexa();//multiplexa displays
    temp = (Ow_Read() << 8) + temp; //le temperatura (byte2)
    multiplexa(); //multiplexa displays

    //--- Formata e mostra nos displays de 7 seguimentos nodo comum
    Mostra_Temp(temp);
    delay_mpx(500);//tempo entre converses
 }//for(;;)
}  //main()